unit streamable_model;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, FileUtil, LResources;

type

  { TStreamable }

  TStreamable = class (TComponent)
  protected
    FParent: TComponent;
    procedure OnFindClass(Reader: TReader; const AClassName: string;
      var ComponentClass: TComponentClass); virtual; abstract;
  public
    procedure SetParentComponent(Value: TComponent); override;
    function GetParentComponent: TComponent; override;
    function HasParent: Boolean; override;
    property Parent: TComponent read GetParentComponent write SetParentComponent;
  public
    procedure SaveToStream(AStream: TStream);
    procedure LoadFromStream(AStream: TStream);
    procedure SaveToFile(FileName: String);
    procedure LoadFromFile(FileName: String);
  end;

implementation

{ TStreamable }

procedure TStreamable.SetParentComponent(Value: TComponent);
begin
  FParent := Value;
end;

function TStreamable.GetParentComponent: TComponent;
begin
  Result:=FParent;
end;

function TStreamable.HasParent: Boolean;
begin
  Result:=FParent<>nil;
end;

procedure TStreamable.SaveToStream(AStream: TStream);
begin
  AStream.Position:=0;
  WriteComponentAsTextToStream(AStream, Self);
end;

procedure TStreamable.LoadFromStream(AStream: TStream);
begin
  AStream.Position:=0;
  ReadComponentFromTextStream(AStream, TComponent(Self), @OnFindClass);
end;

procedure TStreamable.SaveToFile(FileName: String);
var
  Stream: TMemoryStream;
begin
  Stream := TMemoryStream.Create;
  SaveToStream(Stream);
  Stream.SaveToFile(FileName);
  Stream.Free;
end;

procedure TStreamable.LoadFromFile(FileName: String);
var
  Stream: TMemoryStream;
begin
  Stream := TMemoryStream.Create;
  Stream.LoadFromFile(FileName);
  LoadFromStream(Stream);
  Stream.Free;
end;

initialization

end.


