unit Unit1;

{$mode objfpc}{$H+}

interface

uses
  Classes, Forms, StdCtrls, LResources, streamable_model, sysutils, Dialogs,
  Controls;

type

  { TChildClass }

  TChildClass = class (TStreamable)
  private
    FList: TStringList;
    FStr: String;
    FValue: Integer;
  protected
    procedure OnFindClass(Reader: TReader; const AClassName: string;
      var ComponentClass: TComponentClass); override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
    property Value: Integer read FValue write FValue;
    property Str: String read FStr write FStr;
    property List: TStringList read FList write FList;
  end;

  { TParentClass }

  TParentClass = class (TStreamable)
  private
    FChild: TChildClass;
    FList: TStringList;
    FStr: String;
    FValue: Integer;
    procedure SetChild(AValue: TChildClass);
  protected
    procedure OnFindClass(Reader: TReader; const AClassName: string;
      var ComponentClass: TComponentClass); override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure GetChildren(Proc: TGetChildProc; Root: TComponent); override;
  published
    property Child: TChildClass read FChild write SetChild;
    property Value: Integer read FValue write FValue;
    property Str: String read FStr write FStr;
    property List: TStringList read FList write FList;
  end;

  { TForm1 }

  TForm1 = class(TForm)
    RandButton: TButton;
    ShowButton: TButton;
    LoadButton: TButton;
    SaveButton: TButton;
    procedure LoadButtonClick(Sender: TObject);
    procedure RandButtonClick(Sender: TObject);
    procedure SaveButtonClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure ShowButtonClick(Sender: TObject);
  private
    Parent1: TParentClass;
  end;

var
  Form1: TForm1;

implementation

{$R *.lfm}

{ TParentClass }

procedure TParentClass.SetChild(AValue: TChildClass);
begin
  if FChild=AValue then Exit;
  FChild:=AValue;
end;

procedure TParentClass.OnFindClass(Reader: TReader; const AClassName: string;
  var ComponentClass: TComponentClass);
begin
  if CompareText(AClassName, 'TParentClass')=0 then
    ComponentClass:=TParentClass
  else if CompareText(AClassName, 'TChildClass')=0 then
    ComponentClass:=TChildClass
  else
    raise Exception.Create(AClassName);
end;

constructor TParentClass.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FList := TStringList.Create;
  FChild := TChildClass.Create(Self);
  FChild.Parent := Self;
  FChild.Name:='Child';
end;

destructor TParentClass.Destroy;
begin
  FList.Free;
  inherited Destroy;
end;

procedure TParentClass.GetChildren(Proc: TGetChildProc; Root: TComponent);
begin
  inherited GetChildren(Proc, Root);
  Proc(FChild);
end;

{ TChildClass }

procedure TChildClass.OnFindClass(Reader: TReader; const AClassName: string;
  var ComponentClass: TComponentClass);
begin
  if CompareText(AClassName, 'TChildClass')=0 then
    ComponentClass:=TChildClass
  else
    raise Exception.Create(AClassName);
end;

constructor TChildClass.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FList := TStringList.Create;
end;

destructor TChildClass.Destroy;
begin
  FList.Free;
  inherited Destroy;
end;

{ TForm1 }

procedure TForm1.FormCreate(Sender: TObject);
begin
  Parent1 := TParentClass.Create(Self);
  Parent1.Name:='Parent1';
end;

procedure TForm1.ShowButtonClick(Sender: TObject);
var
  S1: String;
  S2: String;
begin
  with Parent1 do
  S1 := Format('Value=%d, Str=%s, List=%s, ComponentCount=%d',
    [Value, Str, List.CommaText, ComponentCount]);
  with Parent1.Child do
  S2 := Format('Value=%d, Str=%s, List=%s, ComponentCount=%d',
    [Value, Str, List.CommaText, ComponentCount]);
  ShowMessageFmt('Parent: %s'+LineEnding+'Child: %s', [S1, S2]);
end;

procedure TForm1.SaveButtonClick(Sender: TObject);
var
  AStream: TMemoryStream;
begin
  AStream := TMemoryStream.Create;
  try
    WriteComponentAsTextToStream(AStream, Parent1);
    AStream.SaveToFile('Parent1.txt');
  finally
    AStream.Free;
  end;
end;

procedure TForm1.LoadButtonClick(Sender: TObject);
var
  AStream: TMemoryStream;
begin
  AStream := TMemoryStream.Create;
  try
    AStream.LoadFromFile('Parent1.txt');
    AStream.Position:=0;
    ReadComponentFromTextStream(AStream, TComponent(Parent1), @Parent1.OnFindClass);
  finally
    AStream.Free;
  end;
end;

procedure TForm1.RandButtonClick(Sender: TObject);
begin
  with Parent1 do
  begin
    Value:=Random(100);
    Str:=Char(Random(26)+65) + Char(Random(26)+65);
    List.CommaText:=Char(Random(26)+65) + Char(Random(26)+65);
  end;

  with Parent1.Child do
  begin
    Value:=Random(100);
    Str:=Char(Random(26)+65) + Char(Random(26)+65);
    List.CommaText:=Char(Random(26)+65) + Char(Random(26)+65);
  end;
end;

initialization

RegisterClass(TParentClass);
RegisterClass(TChildClass);

end.


